@@ -17,15 +17,16 @@
 import re
 import sys
 import inspect
-import pickle
 import warnings
 
 import numpy
 
-from tables.utils import SizeType
-from tables.misc.enum import Enum
+from .utils import SizeType
+from .misc.enum import Enum
 
-from tables._past import previous_api
+import pickle
+
+from .exceptions import FlavorWarning
 
 # Public variables
 # ================
@@ -116,7 +117,7 @@ def _abstract_atom_init(deftype, defvalue):
 def _normalize_shape(shape):
     """Check that the `shape` is safe to be used and return it as a tuple."""
 
-    if isinstance(shape, (int, numpy.integer, int)):
+    if isinstance(shape, (numpy.integer, int)):
         if shape < 1:
             raise ValueError("shape value must be greater than 0: %d"
                              % shape)
@@ -127,7 +128,7 @@ def _normalize_shape(shape):
         raise TypeError("shape must be an integer or sequence: %r"
                         % (shape,))
 
-    ## XXX Get from HDF5 library if possible.
+    # XXX Get from HDF5 library if possible.
     # HDF5 does not support ranks greater than 32
     if len(shape) > 32:
         raise ValueError(
@@ -218,7 +219,7 @@ class MetaAtom(type):
 
 # Atom classes
 # ============
-class Atom(object, metaclass=MetaAtom):
+class Atom(metaclass=MetaAtom):
     """Defines the type of atomic cells stored in a dataset.
 
     The meaning of *atomic* is that individual elements of a cell can
@@ -337,7 +338,7 @@ class Atom(object, metaclass=MetaAtom):
             Traceback (most recent call last):
             ...
             ValueError: unknown NumPy scalar type: 'S5'
-            >>> Atom.from_sctype('Float64')
+            >>> Atom.from_sctype('float64')
             Float64Atom(shape=(), dflt=0.0)
 
         """
@@ -359,9 +360,19 @@ class Atom(object, metaclass=MetaAtom):
             >>> import numpy
             >>> Atom.from_dtype(numpy.dtype((numpy.int16, (2, 2))))
             Int16Atom(shape=(2, 2), dflt=0)
-            >>> Atom.from_dtype(numpy.dtype('Float64'))
+            >>> Atom.from_dtype(numpy.dtype('float64'))
             Float64Atom(shape=(), dflt=0.0)
 
+        Note: for easier use in Python 3, where all strings lead to the
+        Unicode dtype, this dtype will also generate a StringAtom. Since
+        this is only viable for strings that are castable as ascii, a
+        warning is issued.
+
+            >>> Atom.from_dtype(numpy.dtype('U20')) # doctest: +SKIP
+            Atom.py:392: FlavorWarning: support for unicode type is very limited,
+                and only works for strings that can be cast as ascii
+            StringAtom(itemsize=20, shape=(), dflt=b'')
+
         """
         basedtype = dtype.base
         if basedtype.names:
@@ -373,6 +384,13 @@ class Atom(object, metaclass=MetaAtom):
         if basedtype.kind == 'S':  # can not reuse something like 'string80'
             itemsize = basedtype.itemsize
             return class_.from_kind('string', itemsize, dtype.shape, dflt)
+        elif basedtype.kind == 'U':
+            # workaround for unicode type (standard string type in Python 3)
+            warnings.warn("support for unicode type is very limited, "
+                          "and only works for strings that can be cast as ascii", FlavorWarning)
+            itemsize = basedtype.itemsize // 4
+            assert str(itemsize) in basedtype.str, "something went wrong in handling unicode."
+            return class_.from_kind('string', itemsize, dtype.shape, dflt)
         # Most NumPy types have direct correspondence with PyTables types.
         return class_.from_type(basedtype.name, dtype.shape, dflt)
 
@@ -473,17 +491,23 @@ class Atom(object, metaclass=MetaAtom):
 
     # Properties
     # ~~~~~~~~~~
-    size = property(
-        lambda self: self.dtype.itemsize,
-        None, None, "Total size in bytes of the atom.")
-    recarrtype = property(
-        lambda self: str(self.dtype.shape) + self.dtype.base.str[1:],
-        None, None, "String type to be used in numpy.rec.array().")
-    ndim = property(
-        lambda self: len(self.shape), None, None,
+    @property
+    def size(self):
+        """Total size in bytes of the atom."""
+        return self.dtype.itemsize
+
+    @property
+    def recarrtype(self):
+        """String type to be used in numpy.rec.array()."""
+        return str(self.dtype.shape) + self.dtype.base.str[1:]
+
+    @property
+    def ndim(self):
         """The number of dimensions of the atom.
 
-        .. versionadded:: 2.4""")
+        .. versionadded:: 2.4"""
+        return len(self.shape)
+
 
     # Special methods
     # ~~~~~~~~~~~~~~~
@@ -576,7 +600,7 @@ class Atom(object, metaclass=MetaAtom):
             args = inspect.getargspec(self.__init__)[0]
         else:
             parameters = signature.parameters
-            args = [arg for arg, p in list(parameters.items())
+            args = [arg for arg, p in parameters.items()
                 if p.kind is p.POSITIONAL_OR_KEYWORD]
 
         return dict((arg, getattr(self, arg)) for arg in args if arg != 'self')
@@ -597,12 +621,14 @@ class StringAtom(Atom):
     """
 
     kind = 'string'
-    itemsize = property(
-        lambda self: self.dtype.base.itemsize,
-        None, None, "Size in bytes of a sigle item in the atom.")
     type = 'string'
     _defvalue = b''
 
+    @property
+    def itemsize(self):
+        "Size in bytes of a sigle item in the atom."
+        return self.dtype.base.itemsize
+
     def __init__(self, itemsize, shape=(), dflt=_defvalue):
         if not hasattr(itemsize, '__int__') or int(itemsize) < 0:
             raise ValueError("invalid item size for kind ``%s``: %r; "
@@ -668,39 +694,6 @@ def _create_numeric_class(baseclass, itemsize):
     return type('%sAtom' % prefix, (baseclass,), classdict)
 
 
-def _generate_integral_classes():
-    """Generate all integral classes."""
-
-    for baseclass in [IntAtom, UIntAtom]:
-        for itemsize in [1, 2, 4, 8]:
-            newclass = _create_numeric_class(baseclass, itemsize)
-            yield newclass
-
-
-def _generate_floating_classes():
-    """Generate all floating classes."""
-
-    itemsizes = [4, 8]
-
-    # numpy >= 1.6
-    if hasattr(numpy, 'float16'):
-        itemsizes.insert(0, 2)
-    if hasattr(numpy, 'float96'):
-        itemsizes.append(12)
-    if hasattr(numpy, 'float128'):
-        itemsizes.append(16)
-
-    for itemsize in itemsizes:
-        newclass = _create_numeric_class(FloatAtom, itemsize)
-        yield newclass
-
-
-# Create all numeric atom classes.
-#for _classgen in [_generate_integral_classes, _generate_floating_classes]:
-#    for _newclass in _classgen():
-#        exec('%s = _newclass' % _newclass.__name__)
-#del _classgen, _newclass
-
 Int8Atom = _create_numeric_class(IntAtom, 1)
 Int16Atom = _create_numeric_class(IntAtom, 2)
 Int32Atom = _create_numeric_class(IntAtom, 4)
@@ -736,13 +729,15 @@ class ComplexAtom(Atom):
     # using the old bottom-level complex classes.
 
     kind = 'complex'
-    itemsize = property(
-        lambda self: self.dtype.base.itemsize,
-        None, None, "Size in bytes of a sigle item in the atom.")
     _deftype = 'complex128'
     _defvalue = 0j
     _isizes = [8, 16]
 
+    @property
+    def itemsize(self):
+         "Size in bytes of a sigle item in the atom."
+         return self.dtype.base.itemsize
+
     # Only instances have a `type` attribute, so complex types must be
     # registered by hand.
     all_types.add('complex64')
@@ -912,9 +907,10 @@ class EnumAtom(Atom):
 
     # Properties
     # ~~~~~~~~~~
-    itemsize = property(
-        lambda self: self.dtype.base.itemsize,
-        None, None, "Size in bytes of a sigle item in the atom.")
+    @property
+    def itemsize(self):
+        """Size in bytes of a single item in the atom."""
+        return self.dtype.base.itemsize
 
     # Private methods
     # ~~~~~~~~~~~~~~~
@@ -953,7 +949,6 @@ class EnumAtom(Atom):
             raise NotImplementedError("only scalar concrete values "
                                       "are supported for the moment, sorry")
 
-    _checkBase = previous_api(_checkbase)
 
     def _get_init_args(self):
         """Get a dictionary of instance constructor arguments."""
@@ -1013,6 +1008,30 @@ class EnumAtom(Atom):
     #    return hash((self.__class__, self.enum, self.shape, self.dflt,
     #                 self.base))
 
+
+class ReferenceAtom(Atom):
+    """Defines an atom of type object to read references.
+    This atom is read-only.
+    """
+
+    kind = 'reference'
+    type = 'object'
+    _deftype = 'NoneType'
+    _defvalue = None
+
+    # Properties
+    # ~~~~~~~~~~
+    @property
+    def itemsize(self):
+        """Size in bytes of a single item in the atom."""
+        return self.dtype.base.itemsize
+
+    def __init__(self, shape=()):
+        Atom.__init__(self, self.type, shape, self._defvalue)
+
+    def __repr__(self):
+        return 'ReferenceAtom(shape=%s)' % (self.shape,)
+
 # Pseudo-atom classes
 # ===================
 #
@@ -1085,7 +1104,7 @@ class VLStringAtom(_BufferedAtom):
     you will need to *explicitly* convert it to a bytestring before you can
     save them::
 
-        >>> s = 'A unicode string: hbar = \\u210f'
+        >>> s = 'A unicode string: hbar = \u210f'
         >>> bytestring = s.encode('utf-8')
         >>> VLArray.append(bytestring) # doctest: +SKIP
 
@@ -1111,7 +1130,7 @@ class VLStringAtom(_BufferedAtom):
         return numpy.string_(object_)
 
     def fromarray(self, array):
-        return array.tostring()
+        return array.tobytes()
 
 
 class VLUnicodeAtom(_BufferedAtom):
@@ -1137,24 +1156,23 @@ class VLUnicodeAtom(_BufferedAtom):
     type = 'vlunicode'
     base = UInt32Atom()
 
-    if sys.version_info[0] > 2 or sys.maxunicode <= 0xffff:
-        # numpy.unicode_ no more implements the buffer interface in Python 3
-        #
-        # When the Python build is UCS-2, we need to promote the
-        # Unicode string to UCS-4.  We *must* use a 0-d array since
-        # NumPy scalars inherit the UCS-2 encoding from Python (see
-        # NumPy ticket #525).  Since ``_tobuffer()`` can't return an
-        # array, we must override ``toarray()`` itself.
-        def toarray(self, object_):
-            if isinstance(object_, bytes):
-                warnings.warn("Storing bytestrings in VLUnicodeAtom is "
-                              "deprecated.", DeprecationWarning)
-            elif not isinstance(object_, str):
-                raise TypeError("object is not a string: %r" % (object_,))
-            ustr = str(object_)
-            uarr = numpy.array(ustr, dtype='U')
-            return numpy.ndarray(
-                buffer=uarr, dtype=self.base.dtype, shape=len(ustr))
+    # numpy.unicode_ no more implements the buffer interface in Python 3
+    #
+    # When the Python build is UCS-2, we need to promote the
+    # Unicode string to UCS-4.  We *must* use a 0-d array since
+    # NumPy scalars inherit the UCS-2 encoding from Python (see
+    # NumPy ticket #525).  Since ``_tobuffer()`` can't return an
+    # array, we must override ``toarray()`` itself.
+    def toarray(self, object_):
+        if isinstance(object_, bytes):
+            warnings.warn("Storing bytestrings in VLUnicodeAtom is "
+                          "deprecated.", DeprecationWarning)
+        elif not isinstance(object_, str):
+            raise TypeError("object is not a string: %r" % (object_,))
+        ustr = str(object_)
+        uarr = numpy.array(ustr, dtype='U')
+        return numpy.ndarray(
+            buffer=uarr, dtype=self.base.dtype, shape=len(ustr))
 
     def _tobuffer(self, object_):
         # This works (and is used) only with UCS-4 builds of Python,
@@ -1170,7 +1188,7 @@ class VLUnicodeAtom(_BufferedAtom):
     def fromarray(self, array):
         length = len(array)
         if length == 0:
-            return ''  # ``array.view('U0')`` raises a `TypeError`
+            return u''  # ``array.view('U0')`` raises a `TypeError`
         return array.view('U%d' % length).item()
 
 
@@ -1203,4 +1221,4 @@ class ObjectAtom(_BufferedAtom):
         # record when in fact it is empty.
         if array.size == 0:
             return None
-        return pickle.loads(array.tostring())
+        return pickle.loads(array.tobytes())
